"""
Some codes from https://github.com/Newmu/dcgan_code
"""
from __future__ import division
import math
import json
import random
import pprint
import scipy.misc
import numpy as np
from time import gmtime, strftime
from six.moves import xrange
from glob import glob
import os

import tensorflow as tf
import tensorflow.contrib.slim as slim

#from ops import *
import ops

pp = pprint.PrettyPrinter()

get_stddev = lambda x, k_h, k_w: 1/math.sqrt(k_w*k_h*x.get_shape()[-1])

def show_all_variables():
  model_vars = tf.trainable_variables()
  slim.model_analyzer.analyze_vars(model_vars, print_info=True)

def get_image(image_path, input_height, input_width,
              resize_height=64, resize_width=64,
              crop=True, grayscale=False):
  image = imread(image_path, grayscale)
  return transform(image, input_height, input_width,
                   resize_height, resize_width, crop)

def save_images(images, size, image_path):
  return imsave(inverse_transform(images), size, image_path)

def imread(path, grayscale = False):
  if (grayscale):
    return scipy.misc.imread(path, flatten = True).astype(np.float)
  else:
    return scipy.misc.imread(path).astype(np.float)

def merge_images(images, size):
  return inverse_transform(images)

def merge(images, size):
  h, w = images.shape[1], images.shape[2]
  if (images.shape[3] in (3,4)):
    c = images.shape[3]
    img = np.zeros((h * size[0], w * size[1], c))
    for idx, image in enumerate(images):
      i = idx % size[1]
      j = idx // size[1]
      img[j * h:j * h + h, i * w:i * w + w, :] = image
    return img
  elif images.shape[3]==1:
    img = np.zeros((h * size[0], w * size[1]))
    for idx, image in enumerate(images):
      i = idx % size[1]
      j = idx // size[1]
      img[j * h:j * h + h, i * w:i * w + w] = image[:,:,0]
    return img
  else:
    raise ValueError('in merge(images,size) images parameter '
                     'must have dimensions: HxW or HxWx3 or HxWx4')

def imsave(images, size, path):
  image = np.squeeze(merge(images, size))
  return scipy.misc.imsave(path, image)

def center_crop(x, crop_h, crop_w,
                resize_h=64, resize_w=64):
  if crop_w is None:
    crop_w = crop_h
  h, w = x.shape[:2]
  j = int(round((h - crop_h)/2.))
  i = int(round((w - crop_w)/2.))
  return scipy.misc.imresize(
      x[j:j+crop_h, i:i+crop_w], [resize_h, resize_w])

def transform(image, input_height, input_width, 
              resize_height=64, resize_width=64, crop=True):
  if crop:
    cropped_image = center_crop(
      image, input_height, input_width, 
      resize_height, resize_width)
  else:
    cropped_image = scipy.misc.imresize(image, [resize_height, resize_width])
  return np.array(cropped_image)/127.5 - 1.

def inverse_transform(images):
  return (images+1.)/2.

def to_json(output_path, *layers):
  with open(output_path, "w") as layer_f:
    lines = ""
    for w, b, bn in layers:
      layer_idx = w.name.split('/')[0].split('h')[1]

      B = b.eval()

      if "lin/" in w.name:
        W = w.eval()
        depth = W.shape[1]
      else:
        W = np.rollaxis(w.eval(), 2, 0)
        depth = W.shape[0]

      biases = {"sy": 1, "sx": 1, "depth": depth, "w": ['%.2f' % elem for elem in list(B)]}
      if bn != None:
        gamma = bn.gamma.eval()
        beta = bn.beta.eval()

        gamma = {"sy": 1, "sx": 1, "depth": depth, "w": ['%.2f' % elem for elem in list(gamma)]}
        beta = {"sy": 1, "sx": 1, "depth": depth, "w": ['%.2f' % elem for elem in list(beta)]}
      else:
        gamma = {"sy": 1, "sx": 1, "depth": 0, "w": []}
        beta = {"sy": 1, "sx": 1, "depth": 0, "w": []}

      if "lin/" in w.name:
        fs = []
        for w in W.T:
          fs.append({"sy": 1, "sx": 1, "depth": W.shape[0], "w": ['%.2f' % elem for elem in list(w)]})

        lines += """
          var layer_%s = {
            "layer_type": "fc", 
            "sy": 1, "sx": 1, 
            "out_sx": 1, "out_sy": 1,
            "stride": 1, "pad": 0,
            "out_depth": %s, "in_depth": %s,
            "biases": %s,
            "gamma": %s,
            "beta": %s,
            "filters": %s
          };""" % (layer_idx.split('_')[0], W.shape[1], W.shape[0], biases, gamma, beta, fs)
      else:
        fs = []
        for w_ in W:
          fs.append({"sy": 5, "sx": 5, "depth": W.shape[3], "w": ['%.2f' % elem for elem in list(w_.flatten())]})

        lines += """
          var layer_%s = {
            "layer_type": "deconv", 
            "sy": 5, "sx": 5,
            "out_sx": %s, "out_sy": %s,
            "stride": 2, "pad": 1,
            "out_depth": %s, "in_depth": %s,
            "biases": %s,
            "gamma": %s,
            "beta": %s,
            "filters": %s
          };""" % (layer_idx, 2**(int(layer_idx)+2), 2**(int(layer_idx)+2),
               W.shape[0], W.shape[3], biases, gamma, beta, fs)
    layer_f.write(" ".join(lines.replace("'","").split()))

def make_gif(images, fname, duration=2, true_image=False):
  import moviepy.editor as mpy

  def make_frame(t):
    try:
      x = images[int(len(images)/duration*t)]
    except:
      x = images[-1]

    if true_image:
      return x.astype(np.uint8)
    else:
      return ((x+1)/2*255).astype(np.uint8)

  clip = mpy.VideoClip(make_frame, duration=duration)
  clip.write_gif(fname, fps = len(images) / duration)

def visualize(sess, dcgan, config, option):
  image_frame_dim = int(math.ceil(config.batch_size**.5))
  if option == 0:
    z_sample = np.random.uniform(-0.5, 0.5, size=(config.batch_size, dcgan.z_dim))
    samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample})
    save_images(samples, [image_frame_dim, image_frame_dim], './test_laji.png')

    #save_images(samples, [image_frame_dim, image_frame_dim], './samples/test_%s.png' % strftime("%Y%m%d%H%M%S", gmtime()))
  elif option == 1:
    values = np.arange(0, 1, 1./config.batch_size)
    for idx in xrange(100):
      print(" [*] %d" % idx)
      z_sample = np.zeros([config.batch_size, dcgan.z_dim])
      for kdx, z in enumerate(z_sample):
        z[idx] = values[kdx]

      if config.dataset == "mnist":
        y = np.random.choice(10, config.batch_size)
        y_one_hot = np.zeros((config.batch_size, 10))
        y_one_hot[np.arange(config.batch_size), y] = 1

        samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample, dcgan.y: y_one_hot})
      else:
        samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample})

      save_images(samples, [image_frame_dim, image_frame_dim], './samples/test_arange_%s.png' % (idx))
  elif option == 2:
    values = np.arange(0, 1, 1./config.batch_size)
    for idx in [random.randint(0, 99) for _ in xrange(100)]:
      print(" [*] %d" % idx)
      z = np.random.uniform(-0.2, 0.2, size=(dcgan.z_dim))
      z_sample = np.tile(z, (config.batch_size, 1))
      #z_sample = np.zeros([config.batch_size, dcgan.z_dim])
      for kdx, z in enumerate(z_sample):
        z[idx] = values[kdx]

      if config.dataset == "mnist":
        y = np.random.choice(10, config.batch_size)
        y_one_hot = np.zeros((config.batch_size, 10))
        y_one_hot[np.arange(config.batch_size), y] = 1

        samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample, dcgan.y: y_one_hot})
      else:
        samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample})

      try:
        make_gif(samples, './samples/test_gif_%s.gif' % (idx))
      except:
        save_images(samples, [image_frame_dim, image_frame_dim], './samples/test_%s.png' % strftime("%Y%m%d%H%M%S", gmtime()))
  elif option == 3:
    values = np.arange(0, 1, 1./config.batch_size)
    for idx in xrange(100):
      print(" [*] %d" % idx)
      z_sample = np.zeros([config.batch_size, dcgan.z_dim])
      for kdx, z in enumerate(z_sample):
        z[idx] = values[kdx]

      samples = sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample})
      make_gif(samples, './samples/test_gif_%s.gif' % (idx))
  elif option == 4:
    image_set = []
    values = np.arange(0, 1, 1./config.batch_size)

    for idx in xrange(100):
      print(" [*] %d" % idx)
      z_sample = np.zeros([config.batch_size, dcgan.z_dim])
      for kdx, z in enumerate(z_sample): z[idx] = values[kdx]

      image_set.append(sess.run(dcgan.sampler, feed_dict={dcgan.z: z_sample}))
      make_gif(image_set[-1], './samples/test_gif_%s.gif' % (idx))

    new_image_set = [merge(np.array([images[idx] for images in image_set]), [10, 10]) \
        for idx in range(64) + range(63, -1, -1)]
    make_gif(new_image_set, './samples/test_gif_merged.gif', duration=8)

  elif option == 5:
      dcgan.data = glob(os.path.join(
        "./data", config.dataset, dcgan.input_fname_pattern))
      idx_num = len(dcgan.data)
      
      image_dims = [dcgan.input_height, dcgan.input_width, dcgan.c_dim]
      inputs = tf.placeholder(tf.float32, [dcgan.batch_size] + image_dims, name='real_images')
      
      DD = [];
      DD2 = [];
      
      idx_num = 5000
      batch_idxs = idx_num // config.batch_size

      for idx in xrange(0, batch_idxs):
          batch_files = dcgan.data[idx*config.batch_size:(idx+1)*config.batch_size]
          batch = [
            get_image(batch_file,
                  input_height=dcgan.input_height,
                  input_width=dcgan.input_width,
                  resize_height=dcgan.output_height,
                  resize_width=dcgan.output_width,
                  crop=dcgan.crop,
                  grayscale=dcgan.grayscale) for batch_file in batch_files]
          if dcgan.grayscale:
              batch_images = np.array(batch).astype(np.float32)[:, :, :, None]
          else:
              batch_images = np.array(batch).astype(np.float32)
          print(idx, " ", batch_images.shape)
          D, D_logits = sess.run(dcgan.discriminator(inputs, reuse=True), feed_dict={inputs: batch_images})
          
          DD[idx*config.batch_size:(idx+1)*config.batch_size] = D
          DD2[idx*config.batch_size:(idx+1)*config.batch_size] = D_logits
      
      OutputFile = open('./results/discrimination_index_'+config.dataset+'.txt', 'wb')
      np.savetxt(OutputFile, DD, fmt='%.5f')
      OutputFile2 = open('./results/discrimination_index_logits_'+config.dataset+'.txt', 'wb')
      np.savetxt(OutputFile2, DD2, fmt='%.5f')
      
  elif option ==6: # for test
      dcgan.data = glob(os.path.join(
        "./data", config.dataset, dcgan.input_fname_pattern))
      
      idx_num = len(dcgan.data)
      OutputFile = open('./results/simulated_labels_'+config.dataset+'.txt', 'w')
      
      for idx in xrange(0, idx_num):
#      for idx in xrange(0, 10):
          batch_files = dcgan.data[idx:(idx + 1)]
          print(batch_files)
          OutputFile.write(str(batch_files) + '\r\n')
          
  elif option ==7: # for test chunhui launchpad
      dcgan.data = glob(os.path.join(
        "./data", config.dataset, dcgan.input_fname_pattern))
      
      idx_num = len(dcgan.data)
      OutputFile = open('./launchpad_ch/python_'+config.dataset+'.txt', 'w')
      
#      for idx in xrange(0, idx_num):
      for idx in xrange(0, 10):
          batch_files = dcgan.data[idx:(idx + 1)]
          print(batch_files)
          OutputFile.write(str(batch_files) + '\r\n')
  elif option ==8: # for test loss_func_masked
        print('test the loss function masked')
        iterations = 100 #500
        random_z_num = 1

        dcgan.data = glob(os.path.join(
        "./test_set", config.dataset, dcgan.input_fname_pattern))
#        batch_idxs = min(len(dcgan.data), config.train_size) // config.batch_size

        mask_image = [get_image('./Masks/mask_'+config.mask+'.png',
                          input_height=dcgan.input_height,
                          input_width=dcgan.input_width,
                          resize_height=dcgan.output_height,
                          resize_width=dcgan.output_width,
                          crop=dcgan.crop,
                          grayscale='False')]
#        print('mask_image', mask_image)
        mask_image2 = np.array(mask_image).astype(np.float32)
        print('mask_image2', mask_image2)
        tmp_t = sess.run(tf.reduce_sum(mask_image2))
        print('sum_mask_image2', tmp_t)
        mask_image3 = sess.run(tf.nn.relu(mask_image2))
        print('mask_image3', mask_image3)
        tmp_t = sess.run(tf.reduce_sum(mask_image3))
        print('sum_mask_image2', tmp_t)
        
        mask_image2 = mask_image3

        idx_num = len(dcgan.data)
        
        
        idx = config.count - 1
#        for idx in xrange(0, idx_num):
#        for idx in xrange(0, 5000):
        # Define loaded image
        print('the idx number is: ', idx)
        batch_files = dcgan.data[idx:(idx + 1)]
        print(batch_files[0])
        batch = [
          get_image(batch_file,
                input_height=dcgan.input_height,
                input_width=dcgan.input_width,
                resize_height=dcgan.output_height,
                resize_width=dcgan.output_width,
                crop=dcgan.crop,
                grayscale=dcgan.grayscale) for batch_file in batch_files]
        if dcgan.grayscale:
          batch_images = np.array(batch).astype(np.float32)[:, :, :, None]
        else:
          batch_images = np.array(batch).astype(np.float32)          
          
        if dcgan.grayscale:
          scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
        else:
          scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
     

        for start in xrange(0, random_z_num): 
          ini_z = np.random.uniform(-1, 1, size=(1, dcgan.z_dim)) \
                                   .astype(np.float32)
#            print("THE SIZE OF INI_Z IS:", ini_z.shape)
          zz = tf.Variable(ini_z, name='zz', dtype=tf.float32)
          generated_image = sampler3(dcgan, zz)
          print(generated_image.shape)
          loss = loss_func_masked(generated_image, batch_images, mask_image2)
          optimizer = tf.train.GradientDescentOptimizer(0.000002).minimize(loss, var_list=zz)

          sess.run(zz.initializer)
          image = sess.run(generated_image)
          if dcgan.grayscale:
              scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_begin_'
                            + 'idx'+str(idx)+'_'+'z'+str(start)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
              scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_begin_'
                            + 'idx'+str(idx)+'_'+'z'+str(start)+'_'+'iter'+str(iterations)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])

          OutputFile = open('./test/'+config.dataset+'/'+config.mask+'/iloss_func_'
                              + 'idx'+str(idx)+'_'+'z'+str(start)+'_'+'iter'+str(iterations)+'.txt', 'w')
            
            

          for step in range(iterations):
            print(sess.run(zz))
#              print("GENERARED_IMAGE:", generated_image.eval())
            loss_out = sess.run(loss)
#              print('loss', loss.eval())
            print('loss_out', loss_out)
              #sumup.append(loss.eval())
            OutputFile.write(str(loss_out) + '\r\n')
            sess.run(optimizer)
            new_image = sess.run(generated_image)
#              print('generated_image shape', generated_image.shape)

          if dcgan.grayscale:
              scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_' + 'idx'+str(idx)+'_' +
                            'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
              scipy.misc.imsave('./test/'+config.dataset+'/'+config.mask+'/image_' + 'idx'+str(idx)+'_' +
                            'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
                
          OutputFile.close()




  elif option == 11:
        # Define generated image
        iterations = 500
        random_z_num = 1 #10

        dcgan.data = glob(os.path.join(
        "./test_set", config.dataset, dcgan.input_fname_pattern))
#        batch_idxs = min(len(dcgan.data), config.train_size) // config.batch_size

        idx_num = len(dcgan.data)
        
#        for idx in xrange(0, idx_num):
        for idx in xrange(0, 5000):
        # Define loaded image
          batch_files = dcgan.data[idx:(idx + 1)]
          print(batch_files[0])
          batch = [
            get_image(batch_file,
                  input_height=dcgan.input_height,
                  input_width=dcgan.input_width,
                  resize_height=dcgan.output_height,
                  resize_width=dcgan.output_width,
                  crop=dcgan.crop,
                  grayscale=dcgan.grayscale) for batch_file in batch_files]
          if dcgan.grayscale:
            batch_images = np.array(batch).astype(np.float32)[:, :, :, None]
          else:
            batch_images = np.array(batch).astype(np.float32)
            
          if dcgan.grayscale:
            scipy.misc.imsave('./results/'+config.dataset+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
            scipy.misc.imsave('./results/'+config.dataset+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
     

          for start in xrange(0, random_z_num): 
            ini_z = np.random.uniform(-1, 1, size=(1, dcgan.z_dim)) \
                                     .astype(np.float32)
            print("THE SIZE OF INI_Z IS:", ini_z.shape)
            zz = tf.Variable(ini_z, name='zz', dtype=tf.float32)
            generated_image = sampler3(dcgan, zz)
            loss = loss_func(generated_image, batch_images)
            optimizer = tf.train.GradientDescentOptimizer(0.000002).minimize(loss, var_list=zz)

            sess.run(zz.initializer)
            image = sess.run(generated_image)
            if dcgan.grayscale:
                scipy.misc.imsave('./results/'+config.dataset+'/image_begin_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
            else:
                scipy.misc.imsave('./results/'+config.dataset+'/image_begin_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])

            OutputFile = open('./results/'+config.dataset+'/iloss_func_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'_'+'iter'+str(iterations)+'.txt', 'w')

            for step in range(iterations):
              print(sess.run(zz))
#              print("GENERARED_IMAGE:", generated_image.eval())
              # sess.run(img_loss_mean)
              # print("img_loss_mean", img_loss_mean.eval())
              loss_out = sess.run(loss)
#              print('loss', loss.eval())
              print('loss_out', loss_out)
              #sumup.append(loss.eval())
              OutputFile.write(str(loss_out) + '\r\n')
              sess.run(optimizer)
              new_image = sess.run(generated_image)
#              print('generated_image shape', generated_image.shape)
              #save_images(image_new, [1, 1], './test23333.png')
              # make_gif(image_new, './test.gif')
              #scipy.misc.imsave('image_new2.png', image_new[0, 0:256, 0:176, 0])
            
            if dcgan.grayscale:
                scipy.misc.imsave('./results/'+config.dataset+'/image_' + 'idx'+str(idx)+'_' +
                              'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
            else:
                scipy.misc.imsave('./results/'+config.dataset+'/image_' + 'idx'+str(idx)+'_' +
                              'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
                
            OutputFile.close()

  elif option == 12: # for the condition of eliminating images
        # Define generated image
        iterations = 500
        random_z_num = 1

        dcgan.data = glob(os.path.join(
        "./test_set", config.dataset, dcgan.input_fname_pattern))
#        batch_idxs = min(len(dcgan.data), config.train_size) // config.batch_size

        mask_image = [get_image("./eliminated_mask_frontal.png", # NEED TO CHANGE!
                          input_height=dcgan.input_height,
                          input_width=dcgan.input_width,
                          resize_height=dcgan.output_height,
                          resize_width=dcgan.output_width,
                          crop=dcgan.crop,
                          grayscale='True')]
        mask_image2 = np.array(mask_image).astype(np.float32)
#        print(mask_image2.shape)

        idx_num = len(dcgan.data)
        
        
        
#        for idx in xrange(0, idx_num):
        for idx in xrange(0, 5000):
        # Define loaded image
          batch_files = dcgan.data[idx:(idx + 1)]
          print(batch_files[0])
          batch = [
            get_image(batch_file,
                  input_height=dcgan.input_height,
                  input_width=dcgan.input_width,
                  resize_height=dcgan.output_height,
                  resize_width=dcgan.output_width,
                  crop=dcgan.crop,
                  grayscale=dcgan.grayscale) for batch_file in batch_files]
          if dcgan.grayscale:
            batch_images = np.array(batch).astype(np.float32)[:, :, :, None]
          else:
            batch_images = np.array(batch).astype(np.float32)
            
          if dcgan.grayscale:
            scipy.misc.imsave('./results/'+config.dataset+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
            scipy.misc.imsave('./results/'+config.dataset+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
     

          for start in xrange(0, random_z_num): 
            ini_z = np.random.uniform(-1, 1, size=(1, dcgan.z_dim)) \
                                     .astype(np.float32)
#            print("THE SIZE OF INI_Z IS:", ini_z.shape)
            zz = tf.Variable(ini_z, name='zz', dtype=tf.float32)
            generated_image = sampler3(dcgan, zz)
            print(generated_image.shape)
            loss = loss_func_masked(generated_image, batch_images, mask_image2)
            optimizer = tf.train.GradientDescentOptimizer(0.000002).minimize(loss, var_list=zz)

            sess.run(zz.initializer)
            image = sess.run(generated_image)
            if dcgan.grayscale:
                scipy.misc.imsave('./results/'+config.dataset+'/image_begin_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
            else:
                scipy.misc.imsave('./results/'+config.dataset+'/image_begin_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'_'+'iter'+str(iterations)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])

            OutputFile = open('./results/'+config.dataset+'/iloss_func_'
                              + 'idx'+str(idx)+'_'+'zz'+str(start)+'_'+'iter'+str(iterations)+'.txt', 'w')
            
            

            for step in range(iterations):
              print(sess.run(zz))
#              print("GENERARED_IMAGE:", generated_image.eval())
              loss_out = sess.run(loss)
#              print('loss', loss.eval())
              print('loss_out', loss_out)
              #sumup.append(loss.eval())
              OutputFile.write(str(loss_out) + '\r\n')
              sess.run(optimizer)
              new_image = sess.run(generated_image)
#              print('generated_image shape', generated_image.shape)
            
            if dcgan.grayscale:
                scipy.misc.imsave('./results/'+config.dataset+'/image_' + 'idx'+str(idx)+'_' +
                              'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
            else:
                scipy.misc.imsave('./results/'+config.dataset+'/image_' + 'idx'+str(idx)+'_' +
                              'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
                
            OutputFile.close()

  elif option == 13: # for the condition of launchpad for option 12
        # Define generated image
        iterations = 1200
        random_z_num = 1

        dcgan.data = glob(os.path.join(
        "./test_set", config.dataset, dcgan.input_fname_pattern))
#        batch_idxs = min(len(dcgan.data), config.train_size) // config.batch_size

        mask_image = [get_image('./Masks/mask_'+config.mask+'.png',
                          input_height=dcgan.input_height,
                          input_width=dcgan.input_width,
                          resize_height=dcgan.output_height,
                          resize_width=dcgan.output_width,
                          crop=dcgan.crop,
                          grayscale='False')]
#        print('mask_image', mask_image)
        mask_image2 = np.array(mask_image).astype(np.float32)
        print('mask_image2', mask_image2)
#        tmp_t = sess.run(tf.reduce_sum(mask_image2))
#        print('sum_mask_image2', tmp_t)
        mask_image3 = sess.run(tf.nn.relu(mask_image2))
        print('mask_image3', mask_image3)
#        tmp_t = sess.run(tf.reduce_sum(mask_image3))
#        print('sum_mask_image2', tmp_t)
        mask_image2 = mask_image3




        idx_num = len(dcgan.data)
        
        
        idx = config.count - 1
#        for idx in xrange(0, idx_num):
#        for idx in xrange(0, 5000):
        # Define loaded image
        print('the idx number is: ', idx)
        batch_files = dcgan.data[idx:(idx + 1)]
        print(batch_files[0])
        batch = [
          get_image(batch_file,
                input_height=dcgan.input_height,
                input_width=dcgan.input_width,
                resize_height=dcgan.output_height,
                resize_width=dcgan.output_width,
                crop=dcgan.crop,
                grayscale=dcgan.grayscale) for batch_file in batch_files]
        if dcgan.grayscale:
          batch_images = np.array(batch).astype(np.float32)[:, :, :, None]
        else:
          batch_images = np.array(batch).astype(np.float32)          
          
        if dcgan.grayscale:
          scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
        else:
          scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_raw_'+'idx'+str(idx)+'.png', batch_images[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
     

        for start in xrange(0, random_z_num): 
          ini_z = np.random.uniform(-1, 1, size=(1, dcgan.z_dim)) \
                                   .astype(np.float32)
#            print("THE SIZE OF INI_Z IS:", ini_z.shape)
          zz = tf.Variable(ini_z, name='zz', dtype=tf.float32)
          generated_image = sampler3(dcgan, zz)
          print(generated_image.shape)
          loss = loss_func_masked(generated_image, batch_images, mask_image2)
          optimizer = tf.train.GradientDescentOptimizer(0.000002).minimize(loss, var_list=zz)

          sess.run(zz.initializer)
          image = sess.run(generated_image)
          if dcgan.grayscale:
              scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_begin_'
                            + 'idx'+str(idx)+'_'+'z'+str(start)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
              scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_begin_'
                            + 'idx'+str(idx)+'_'+'z'+str(start)+'_'+'iter'+str(iterations)+'.png', image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])

          OutputFile = open('./results/'+config.dataset+'/'+config.mask+'/iloss_func_'
                              + 'idx'+str(idx)+'_'+'z'+str(start)+'_'+'iter'+str(iterations)+'.txt', 'w')
            
            

          for step in range(iterations):
            print(sess.run(zz))
#              print("GENERARED_IMAGE:", generated_image.eval())
            loss_out = sess.run(loss)
#              print('loss', loss.eval())
            print('loss_out', loss_out)
              #sumup.append(loss.eval())
            OutputFile.write(str(loss_out) + '\r\n')
            sess.run(optimizer)
            new_image = sess.run(generated_image)
#              print('generated_image shape', generated_image.shape)

          if dcgan.grayscale:
              scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_' + 'idx'+str(idx)+'_' +
                            'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0])
          else:
              scipy.misc.imsave('./results/'+config.dataset+'/'+config.mask+'/image_' + 'idx'+str(idx)+'_' +
                            'z'+str(start)+'_'+'iter'+str(iterations)+'.png', new_image[0, 0:dcgan.output_height, 0:dcgan.output_width, 0:3])
                
          OutputFile.close()


def image_manifold_size(num_images):
  manifold_h = int(np.floor(np.sqrt(num_images)))
  manifold_w = int(np.ceil(np.sqrt(num_images)))
  assert manifold_h * manifold_w == num_images
  return manifold_h, manifold_w

def sigmoid_cross_entropy_with_logits(x, y):
  try:
    return tf.nn.sigmoid_cross_entropy_with_logits(logits=x, labels=y)
  except:
    return tf.nn.sigmoid_cross_entropy_with_logits(logits=x, targets=y)

def sampler2(dcgan, zz):
    with tf.variable_scope("generator") as scope:
      scope.reuse_variables()

      s_h, s_w = dcgan.output_height, dcgan.output_width
      s_h2, s_w2 = conv_out_size_same(s_h, 2), conv_out_size_same(s_w, 2)
      s_h4, s_w4 = conv_out_size_same(s_h2, 2), conv_out_size_same(s_w2, 2)
      s_h8, s_w8 = conv_out_size_same(s_h4, 2), conv_out_size_same(s_w4, 2)
      s_h16, s_w16 = conv_out_size_same(s_h8, 2), conv_out_size_same(s_w8, 2)

      # project `z` and reshape
      # zz: (64, 100)
      h0 = tf.reshape(
          ops.linear(zz, dcgan.gf_dim*8*s_h16*s_w16, 'g_h0_lin'),
          [-1, s_h16, s_w16, dcgan.gf_dim * 8])
      print("zz:", zz.shape)  # zz: (64, 100)
      print("linear_input", dcgan.gf_dim*8*s_h16*s_w16)  # linear_input 90112
      print('h0.shape:', h0.get_shape())  # h0.shape: (64, 16, 11, 512)
      print("linear_output", [-1, s_h16, s_w16, dcgan.gf_dim * 8])  # linear_output [-1, 16, 11, 512]
      h0 = tf.nn.relu(dcgan.g_bn0(h0, train=False))
      print("h0 after relu:", h0.shape)  # h0 after relu: (64, 16, 11, 512)

      h1 = ops.deconv2d(h0, [dcgan.batch_size, s_h8, s_w8, dcgan.gf_dim*4], name='g_h1')
      print("h0.shape:", h0.get_shape()[-1])  # h0.shape: 512
      h0_1 = [dcgan.batch_size, s_h8, s_w8, dcgan.gf_dim * 4]
      print("output_shape:", h0_1)  # output_shape: [64, 32, 22, 256]
      print('h0_1:', h0_1[-1])  # output_shape[-1]: 256
      print("h1.shape:", h1.shape)  # h1.shape: (64, 32, 22, 256)
      h1 = tf.nn.relu(dcgan.g_bn1(h1, train=False))
      print("h1.shape after relu", h1.shape)  #h1 after relu: (64, 32, 22, 256)

      h2 = ops.deconv2d(h1, [dcgan.batch_size, s_h4, s_w4, dcgan.gf_dim*2], name='g_h2')
      print("h1.shape:", h1.get_shape()[-1])  # h1.shape: 256
      h1_1 = [dcgan.batch_size, s_h4, s_w4, dcgan.gf_dim * 2]
      print("output_shape:", h1_1)  # output_shape: [64, 64, 44, 128]
      print('h1_1:', h1_1[-1])  # output_shape[-1]: 128
      print("h2.shape:", h2.shape)  # h2.shape: (64, 64, 44, 128)
      h2 = tf.nn.relu(dcgan.g_bn2(h2, train=False))
      print("h2.shape after relu", h2.shape)  # h2.shape after relu (64, 64, 44, 128)

      h3 = ops.deconv2d(h2, [dcgan.batch_size, s_h2, s_w2, dcgan.gf_dim*1], name='g_h3')
      print("h2.shape:", h2.get_shape()[-1])  # h2.shape: 128
      h2_1 = [dcgan.batch_size, s_h2, s_w2, dcgan.gf_dim * 1]
      print("output_shape:", h2_1)  # output_shape: [64, 128, 88, 64]
      print('h2_1:', h2_1[-1])  # output_shape[-1]: 64
      print("h3.shape:", h3.shape)  # h3.shape: (64, 128, 88, 64)
      h3 = tf.nn.relu(dcgan.g_bn3(h3, train=False))
      print("h3.shape after relu", h3.shape)  # h2.shape after relu (64, 128, 88, 64)

      h4 = ops.deconv2d(h3, [dcgan.batch_size, s_h, s_w, dcgan.c_dim], name='g_h4')
      print("h3.shape:", h3.get_shape()[-1])  # h3.shape: 64
      h3_1 = [dcgan.batch_size, s_h, s_w, dcgan.c_dim]
      print("output_shape:", h3_1)  # output_shape: [64, 256, 176, 1]
      print('h3_1:', h3_1[-1])  # output_shape[-1]: 1
      print("h4.shape:", h4.shape)  # h4.shape: (64, 256, 176, 1)

      # new stuff
      h4_1 = tf.nn.tanh(h4)
      print("h4_1.shape", h4_1.shape)  # h4_1.shape (64, 256, 176, 1)
      return tf.nn.tanh(h4)

def sampler3(dcgan, zz):
    with tf.variable_scope("generator") as scope:
      scope.reuse_variables()

      s_h, s_w = dcgan.output_height, dcgan.output_width
      s_h2, s_w2 = conv_out_size_same(s_h, 2), conv_out_size_same(s_w, 2)
      s_h4, s_w4 = conv_out_size_same(s_h2, 2), conv_out_size_same(s_w2, 2)
      s_h8, s_w8 = conv_out_size_same(s_h4, 2), conv_out_size_same(s_w4, 2)
      s_h16, s_w16 = conv_out_size_same(s_h8, 2), conv_out_size_same(s_w8, 2)

      # project `z` and reshape
      # zz: (64, 100)
      h0 = tf.reshape(
          ops.linear(zz, dcgan.gf_dim*8*s_h16*s_w16, 'g_h0_lin'),
          [-1, s_h16, s_w16, dcgan.gf_dim * 8])

      h0 = tf.nn.relu(dcgan.g_bn0(h0, train=False))

      h1 = ops.deconv2d(h0, [1, s_h8, s_w8, dcgan.gf_dim*4], name='g_h1')
      h1 = tf.nn.relu(dcgan.g_bn1(h1, train=False))

      h2 = ops.deconv2d(h1, [1, s_h4, s_w4, dcgan.gf_dim*2], name='g_h2')
      h2 = tf.nn.relu(dcgan.g_bn2(h2, train=False))

      h3 = ops.deconv2d(h2, [1, s_h2, s_w2, dcgan.gf_dim*1], name='g_h3')
      h3 = tf.nn.relu(dcgan.g_bn3(h3, train=False))

      h4 = ops.deconv2d(h3, [1, s_h, s_w, dcgan.c_dim], name='g_h4')

      return tf.nn.tanh(h4)

def loss_func(generated_image, batch_images):
  # a = generated_image[(2, 3, 4, 0)]
  # b = batch_images[(2, 3, 4, 0)]

#  GImg_shape = generated_image.shape
#  RImg_shape = batch_images.shape
#  p = 0.0
  tmp1 = tf.square(generated_image - batch_images)
  tmp2 = tf.reduce_sum(tmp1)

  return tmp2

def loss_func_masked(generated_image, batch_images, mask_image):
    shape = generated_image.shape
    mask = mask_image[..., np.newaxis]
    RedG = generated_image[0:shape[0], 0:shape[1], 0:shape[2], 0]
    GreenG = generated_image[0:shape[0], 0:shape[1], 0:shape[2], 1]
    BlueG = generated_image[0:shape[0], 0:shape[1], 0:shape[2], 2]
    
    RedG1 = RedG[...,np.newaxis]
    GreenG1 = GreenG[...,np.newaxis]
    BlueG1 = BlueG[...,np.newaxis]
    
    RedG_masked = tf.multiply(RedG1, mask)
    GreenG_masked = tf.multiply(GreenG1, mask)
    BlueG_masked = tf.multiply(BlueG1, mask)
    

    generatedImg_masked = tf.concat([RedG_masked, GreenG_masked, BlueG_masked], 3)
    # print('RedG', RedG.shape)
    # print('mask', mask.shape)
    # print('RedG_masked', RedG_masked.shape)
    # print('GreenG', GreenG.shape)
    # print('GreenG_masked', GreenG_masked.shape)
    # print('generatedImg_masked', generatedImg_masked.shape)
    RedB = batch_images[0:shape[0], 0:shape[1], 0:shape[2], 0]
    GreenB = batch_images[0:shape[0], 0:shape[1], 0:shape[2], 1]
    BlueB = batch_images[0:shape[0], 0:shape[1], 0:shape[2], 2]
    
    RedB1 = RedB[...,np.newaxis]
    GreenB1 = GreenB[...,np.newaxis]
    BlueB1 = BlueB[...,np.newaxis]
    
    RedB_masked = tf.multiply(RedB1, mask)
    GreenB_masked = tf.multiply(GreenB1, mask)
    BlueB_masked = tf.multiply(BlueB1, mask)
    
    batchImg_masked = tf.concat([RedB_masked, GreenB_masked, BlueB_masked], 3)
    print('batchImg_masked', batchImg_masked.shape)
    tmp1 = tf.square(generatedImg_masked - batchImg_masked)
    tmp2 = tf.reduce_sum(tmp1)
    
    return tmp2


def conv_out_size_same(size, stride):
  return int(math.ceil(float(size) / float(stride)))
